package com.persagy.apm.diagnose.timerecord.service.impl;

import com.persagy.apm.diagnose.timerecord.dao.CalculateTimeRecordMapper;
import com.persagy.apm.diagnose.timerecord.service.ICalculateTimeRecordService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.diagnose.timerecord.model.*;
import com.persagy.apm.diagnose.timerecord.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 监测指标计算时间(CalculateTimeRecord) service层
 *
 * @author lixing
 * @version V1.0 2021-09-10 00:13:10
 */
@Service
public class CalculateTimeRecordServiceImpl extends ServiceImpl<CalculateTimeRecordMapper, CalculateTimeRecord>
        implements ICalculateTimeRecordService {

//    @Resource(name = "ZillionDao")
//    private CoreDao coredao;

    /**
     * 创建监测指标计算时间
     *
     * @return 监测指标计算时间主键
     * @author lixing
     * @version V1.0 2021-09-10 00:13:10
     */
    @Override
    public String createCalculateTimeRecord(AddCalculateTimeRecordDTO addCalculateTimeRecordDTO) {
        CalculateTimeRecord calculateTimeRecord = ConvertCalculateTimeRecordTool.INSTANCE.convertAddDto2Entity(addCalculateTimeRecordDTO);
        // 设置默认值
        setDefaultValue(calculateTimeRecord);
        save(calculateTimeRecord);
        return null;
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param calculateTimeRecord 监测指标计算时间实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(CalculateTimeRecord calculateTimeRecord) {
//        calculateTimeRecord.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 监测指标计算时间详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-09-10 00:13:10
     */
    @Override
    public CalculateTimeRecord queryCalculateTimeRecordDetail(String id) {
        CalculateTimeRecord calculateTimeRecord = getById(id);
        if (calculateTimeRecord == null) {
            throw new IllegalArgumentException("查看CalculateTimeRecord详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return calculateTimeRecord;
    }

    /**
     * 更新监测指标计算时间
     *
     * @author lixing
     * @version V1.0 2021-09-10 00:13:10
     */
    @Override
    public void updateCalculateTimeRecord(UpdateCalculateTimeRecordDTO updateCalculateTimeRecordDTO) {
        CalculateTimeRecord calculateTimeRecord = getById(updateCalculateTimeRecordDTO.getId());
        calculateTimeRecord = ConvertCalculateTimeRecordTool.INSTANCE.convertUpdateDto2Entity(calculateTimeRecord, updateCalculateTimeRecordDTO);
//        calculateTimeRecord.setModifier(AppContext.getContext().getAccountId());
        updateById(calculateTimeRecord);
    }

    /**
     * 校验监测指标计算时间是否可删除
     *
     * @param id 监测指标计算时间主键
     * @return 监测指标计算时间do类
     * @author lixing
     * @version V1.0 2021-09-10 00:13:10
     */
    public CalculateTimeRecord checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除CalculateTimeRecord时发生异常，主键为空");
        }

        CalculateTimeRecord calculateTimeRecord = getById(id);

        if (calculateTimeRecord == null) {
            throw new IllegalArgumentException("删除CalculateTimeRecord时发生异常，找不到要删除的数据，id:" + id);
        }

        return calculateTimeRecord;
    }

    /**
     * 删除监测指标计算时间
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-09-10 00:13:10
     */
    @Override
    public void deleteCalculateTimeRecord(String id) {
        // 校验是否可删除
        CalculateTimeRecord calculateTimeRecord = checkDeletable(id);
//        calculateTimeRecord.setValid(ValidEnum.FALSE.getType());
        updateById(calculateTimeRecord);
    }

    /**
     * 查询监测指标计算时间
     *
     * @return List<CalculateTimeRecord>
     * @author lixing
     * @version V1.0 2021-09-10 00:13:10
     */
    @Override
    public List<CalculateTimeRecord> queryCalculateTimeRecordList(QueryCalculateTimeRecordDTO queryCalculateTimeRecordDTO) {
        QueryWrapper<CalculateTimeRecord> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq(CalculateTimeRecord.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
//        queryWrapper.orderBy(true, false, CalculateTimeRecord.PROP_CREATIONTIME);

        if (queryCalculateTimeRecordDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryCalculateTimeRecordDTO.getObjId())) {
                queryWrapper.like(CalculateTimeRecord.PROP_OBJ_ID, queryCalculateTimeRecordDTO.getObjId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryCalculateTimeRecordDTO.getMonitorIndicatorId())) {
                queryWrapper.like(CalculateTimeRecord.PROP_MONITOR_INDICATOR_ID, queryCalculateTimeRecordDTO.getMonitorIndicatorId());
            }

            if (queryCalculateTimeRecordDTO.getCalculateLastTime() != null) {
                queryWrapper.eq(CalculateTimeRecord.PROP_CALCULATE_LAST_TIME, queryCalculateTimeRecordDTO.getCalculateLastTime());
            }

        }
        return list(queryWrapper);
    }

    /**
     * 分页查询监测指标计算时间
     *
     * @return IPage<CalculateTimeRecord>
     * @author lixing
     * @version V1.0 2021-09-10 00:13:10
     */
    @Override
    public IPage<CalculateTimeRecord> pageQueryCalculateTimeRecord(PageQueryCalculateTimeRecordDTO pageQueryCalculateTimeRecordDTO) {
        QueryWrapper<CalculateTimeRecord> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq(CalculateTimeRecord.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<CalculateTimeRecord> pageParam = new Page<>(pageQueryCalculateTimeRecordDTO.getPage(), pageQueryCalculateTimeRecordDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryCalculateTimeRecordDTO.getOrders())) {
            // 默认按创建时间倒序排序
//            queryWrapper.orderBy(true, false, CalculateTimeRecord.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryCalculateTimeRecordDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryCalculateTimeRecordDTO.getObjId())) {
            queryWrapper.like(CalculateTimeRecord.PROP_OBJ_ID, pageQueryCalculateTimeRecordDTO.getObjId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryCalculateTimeRecordDTO.getMonitorIndicatorId())) {
            queryWrapper.like(CalculateTimeRecord.PROP_MONITOR_INDICATOR_ID, pageQueryCalculateTimeRecordDTO.getMonitorIndicatorId());
        }

        if (pageQueryCalculateTimeRecordDTO.getCalculateLastTime() != null) {
            queryWrapper.eq(CalculateTimeRecord.PROP_CALCULATE_LAST_TIME, pageQueryCalculateTimeRecordDTO.getCalculateLastTime());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public List<CalculateTimeRecord> queryTimeRecordListByProjectId(String projectId) {
        QueryWrapper<CalculateTimeRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(CalculateTimeRecord.PROJECT_ID, projectId);
        return list(queryWrapper);
    }

    @Override
    public void saveOrUpdateByPk(CalculateTimeRecord update) {
        QueryWrapper<CalculateTimeRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(CalculateTimeRecord.PROJECT_ID, update.getProjectId());
        queryWrapper.eq(CalculateTimeRecord.PROP_OBJ_ID, update.getObjId());
        queryWrapper.eq(CalculateTimeRecord.PROP_MONITOR_INDICATOR_ID, update.getMonitorIndicatorId());
        int count = this.count(queryWrapper);
        if (count == 0) {
            //新建
            this.save(update);
        } else {
            this.update(update, queryWrapper);
        }
    }

}
